////////////////////////////////////////////////////////////////////////
// File:			main.cpp
// Author:		M Shaw
// Platform:	Ticol Tcl
// Website:		http://ticol.kerys.co.uk
//
// Purpose:
// Example C++ calling ticol.dll (A __stdcall DLL)
// Ensure stack is sufficient for recursion (try 30,000-100,000)
// Loading the DLL dynamically is very straightforward
//
// Updates:
// 1.00 03/Mar/2021 First version
// 1.01 18/Jan/2022 Updated
////////////////////////////////////////////////////////////////////////
#pragma comment(linker, "/stack:0x100000")
#include <windows.h>
#include <stdio.h>

int main(int argc, char** argv)
{
  LPSTR ptr=NULL;
  long breakpoint=0;
  long stacktrace=0;
  long autoexec=0;
	long debug=0;
  long errnum=0;

	///////////////////////////////////////////////////////////////////////
  // Example embedded Tcl script:
  // DO NOT indent this multi-line literal!
  // Some compilers may also restrict the size of string literals
  // e.g. MSVC 5.0
	// Use: tcl2c.exe /split to convert any existing Tcl file
	///////////////////////////////////////////////////////////////////////

    LPSTR script="\
proc foo {} {\r\n\
textcolor magenta\r\n\
puts \"Hello from proc '[info procname] at [date]@[time]'\"\r\n\
textcolor\r\n\
}\r\n\
\r\n\
foo\r\n\
";

  // Declare funct*
  LPSTR (__stdcall* ticol_command)(LPSTR s,long,long,long,long,long*);

  HINSTANCE hAPI=LoadLibraryA("ticol.dll");               // Load the DLL
  if(hAPI)                                                // Check handle
  {
    ticol_command=(LPSTR (__stdcall*)(LPSTR,long,long,long,long,long*)) \
        GetProcAddress(hAPI,"ticol_command");             // Load the export
    if(ticol_command)                                     // If successful
    {
      printf("test: Dynamically loaded DLL ticol.dll->ticol_command()\r\n\r\n");

      // Call Tcl ...
      ptr=ticol_command(script,breakpoint,stacktrace,autoexec,debug,&errnum);

      printf("\r\ntest: ticol_command: Returned result:'%s' errorcode:%li\r\n"\
       ,(ptr?ptr:"NULL"),errnum);
    }
    else
    {
      printf("test: Failed to link ticol_command in ticol.dll\r\n");
      return 1;
    }
  }
  else
  {
    printf("test: Failed to load ticol.dll\r\n");
    return 2;
  }
  return 0;
}
////////////////////////////////////////////////////////////////////////